﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Mail;
using System.Text;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using Microsoft.SharePoint.Client;
using SharePointIntegrationWeb.Controller;
using SharePointIntegrationWeb.Model;
using Attachment = System.Net.Mail.Attachment;
using File = System.IO.File;

namespace SharePointIntegrationWeb.Pages
{
    public partial class ApproveMember : System.Web.UI.Page
    {
        readonly MemberController _memberController = new MemberController();
        /// <summary>
        /// Checks if the membertype is "Bestyrelse", else redirect him back to startpage.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Page_PreRender(object sender, EventArgs e)
        {
            string memberType = (string)Session["MemberType"];
            if (memberType != "Bestyrelse")
            {
                Response.Redirect((string)Session["StartPage"]);
            }
        }
        /// <summary>
        /// Calls "LoadMembers();" on page load.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Page_Load(object sender, EventArgs e)
        {
            LoadMembers();
        }
        /// <summary>
        /// Checks for members that are not yet activated. Displays a list of deactivated members.
        /// Calls "ShowNonActivatedMembers" which is a method that creates the markup dynamically to see the members.
        /// </summary>
        private void LoadMembers()
        {
            try
            {
                List<Member> nonActivatedMembers = _memberController.GetNonActivatedMembers();

                if (nonActivatedMembers.Count == 0)
                {
                    infoLabel.Text = "Der er ikke nogle bruger der skal godkendes";
                    infoLabel.Visible = true;
                    UserPanel.Controls.Clear();
                }
                else
                {
                    ShowNonActivatedMembers(nonActivatedMembers);
                    infoLabel.Visible = false;
                }
            }
            catch (Exception ex)
            {
                Response.Write(ex.Message);
                Membersystem.ExceptionUtility.LogException(ex, Request.RawUrl);
            }
        }
        /// <summary>
        /// Creates the markup dynamically for each member that needs activation.
        /// </summary>
        /// <param name="nonActivatedMembers">List of nonacticated members.</param>
        private void ShowNonActivatedMembers(List<Member> nonActivatedMembers)
        {
            UserPanel.Controls.Clear();
            foreach (var member in nonActivatedMembers)
            {
                HtmlAnchor nameLabel = new HtmlAnchor { HRef = "#", InnerText = member.Name };
                nameLabel.Attributes.Add("onclick", "showUser(" + member.Id + ")");
                UserPanel.Controls.Add(nameLabel);

                UserPanel.Controls.Add(new Label { Text = "<br/>" });


                Label emailLabel = new Label { Text = "E-mail: " + member.Email + "<br />" };
                Label birthdayLabel = new Label { Text = "Fødselsdag: " + member.Birthday.Date.ToString("d") + "<br />" };
                Label phoneLabel = new Label { Text = "Telefon: " + member.Phone + "<br />" };
                Label addressLabel = new Label { Text = "Adresse: " + member.Address + "<br />" };
                Label cityLabel = new Label { Text = "By: " + member.City + "<br />" };
                Label zipCodeLabel = new Label { Text = "Postnr: " + member.ZipCode + "<br />" };

                Button acceptButton = new Button();
                acceptButton.Text = "Godkend";
                int memberId = member.Id;
                string memberEmail = member.Email;
                string memberName = member.Name;

                acceptButton.Click += (sender, eventArgs) =>
                {
                    AcceptUser(memberId, memberEmail, memberName);
                    
                };
                Button declineButton = new Button();
                declineButton.Text = "Afvis";
                declineButton.Click += (sender, eventArgs) =>
                {
                    DeclineUser(memberId);
                };

                HtmlGenericControl div = new HtmlGenericControl("div");
                div.ID = member.Id.ToString();
                div.Attributes.Add("class", "collapse");
                div.Controls.Add(emailLabel);
                div.Controls.Add(birthdayLabel);
                div.Controls.Add(phoneLabel);
                div.Controls.Add(addressLabel);
                div.Controls.Add(cityLabel);
                div.Controls.Add(zipCodeLabel);
                div.Controls.Add(acceptButton);
                div.Controls.Add(declineButton);
                div.Controls.Add(new Label { Text = "<br/><br/>" });
                UserPanel.Controls.Add(div);
            }
        }
        /// <summary>
        /// If administrator clicks acceptuser, it will take these parameters and change the "Approved By" in the database
        /// and create a document library for the user. It will also send him a "welcome"-email.
        /// </summary>
        /// <param name="userId">UserID of accepted user</param>
        /// <param name="memberEmail">Email of accepted user</param>
        /// <param name="memberName">Name of accepted user</param>
        public void AcceptUser(int userId, string memberEmail, string memberName)
        {
            string approvedByEmail = (string)Session["UserEmail"];
            string message = _memberController.AcceptUser(userId, approvedByEmail);
            if (message == "Godkendt")
            {
                SendEmail(memberEmail);
                SharePointHelper.CreateMemberList(memberEmail, memberName + " " + userId,
                    (ClientContext) Session["clientContext"]);
            }
            ApprovedLabel.Text = message;
            ApprovedLabel.Visible = true;
            LoadMembers();
        }
        /// <summary>
        /// If he is declined, he will be deleted from the database. This is to delete if any spam occures.
        /// </summary>
        /// <param name="userId"></param>
        public void DeclineUser(int userId)
        {
            string approvedByEmail = (string)Session["UserEmail"];
            string message = _memberController.DeclineUser(userId, approvedByEmail);
            ApprovedLabel.Text = message;
            ApprovedLabel.Visible = true;
            LoadMembers();
        }
        /// <summary>
        /// Sends the email which reads from the welcome pdf file.
        /// </summary>
        /// <param name="email">Email to send the email to</param>
        private void SendEmail(string email)
        {
            string filePath = Server.MapPath("Velkommen-som-medlem-af-K19s-Venner-1.pdf");
            string logoPath = Server.MapPath("../Images/logo.png");

            try
            {
                // http://stackoverflow.com/questions/2550796/reading-pdf-content-with-itextsharp-dll-in-vb-net-or-c-sharp
                StringBuilder text = new StringBuilder();

                if (File.Exists(filePath))
                {
                    PdfReader pdfReader = new PdfReader(filePath);

                    for (int page = 1; page <= pdfReader.NumberOfPages; page++)
                    {
                        ITextExtractionStrategy strategy = new SimpleTextExtractionStrategy();
                        string currentText = PdfTextExtractor.GetTextFromPage(pdfReader, page, strategy);

                        currentText = Encoding.UTF8.GetString(ASCIIEncoding.Convert(Encoding.Default, Encoding.UTF8, Encoding.Default.GetBytes(currentText)));
                        text.Append(currentText);
                    }
                    pdfReader.Close();
                }

                Attachment imageAttachment = new Attachment(logoPath);
                imageAttachment.ContentId = "Logo.png"; ;

                MailMessage mail = new MailMessage("admin@k19svenner.onmicrosoft.com", email);
                SmtpClient client = new SmtpClient
                {
                    Port = 587,
                    EnableSsl = true,
                    DeliveryMethod = SmtpDeliveryMethod.Network,
                    UseDefaultCredentials = false,
                    Credentials = new NetworkCredential("admin@k19svenner.onmicrosoft.com", "Vaqu1823"),
                    Host = "smtp.office365.com"
                };

                mail.Subject = "K19s Venner - Velkommen";
                mail.IsBodyHtml = true;
                mail.Body = @"<img src=""cid:Logo.png""/><br />" + text;
                mail.Attachments.Add(imageAttachment);
                client.Send(mail);
                Response.Write("Info er sendt til brugeren");
                imageAttachment.Dispose();
            }
            catch (Exception ex)
            {
                Response.Write(ex.Message);
            }
        }

        protected void BackButton_Click(object sender, EventArgs e)
        {
            Response.Redirect((string)Session["StartPage"]);
        }
    }
}